@@ -5,11 +5,7 @@ from pathlib import Path
 from code_puppy.command_line.model_picker_completion import update_model_in_input
 from code_puppy.command_line.motd import print_motd
 from code_puppy.command_line.utils import make_directory_table
-from code_puppy.config import (
-    CONTEXTS_DIR,
-    finalize_autosave_session,
-    get_config_keys,
-)
+from code_puppy.config import CONTEXTS_DIR, finalize_autosave_session, get_config_keys
 from code_puppy.session_storage import list_sessions, load_session, save_session
 from code_puppy.tools.tools_content import tools_content
 
@@ -33,7 +29,10 @@ def get_commands_help():
         ("/exit, /quit", "Exit interactive mode"),
         ("/generate-pr-description [@dir]", "Generate comprehensive PR description"),
         ("/model, /m <model>", "Set active model"),
-        ("/reasoning <low|medium|high>", "Set OpenAI reasoning effort for GPT-5 models"),
+        (
+            "/reasoning <low|medium|high>",
+            "Set OpenAI reasoning effort for GPT-5 models",
+        ),
         ("/pin_model <agent> <model>", "Pin a specific model to an agent"),
         ("/mcp", "Manage MCP servers (list, start, stop, status, etc.)"),
         ("/motd", "Show the latest message of the day (MOTD)"),
@@ -44,10 +43,12 @@ def get_commands_help():
         ),
         ("/dump_context <name>", "Save current message history to file"),
         ("/load_context <name>", "Load message history from file"),
+        ("/autosave_load", "Load an autosave session interactively"),
         (
             "/set",
-            "Set puppy config (e.g., /set yolo_mode true, /set auto_save_session true)",
+            "Set puppy config (e.g., /set yolo_mode true, /set auto_save_session true, /set diff_context_lines 10)",
         ),
+        ("/diff", "Configure diff highlighting colors (additions, deletions)"),
         ("/tools", "Show available tools and capabilities"),
         (
             "/truncate <N>",
@@ -132,8 +133,104 @@ def _ensure_plugins_loaded() -> None:
         _PLUGINS_LOADED = True
 
 
+def _show_color_options(color_type: str):
+    """Show available Rich color options organized by category."""
+    from code_puppy.messaging import emit_info
+
+    # Standard Rich colors organized by category
+    color_categories = {
+        "Basic Colors": [
+            ("black", "⚫"),
+            ("red", "🔴"),
+            ("green", "🟢"),
+            ("yellow", "🟡"),
+            ("blue", "🔵"),
+            ("magenta", "🟣"),
+            ("cyan", "🔷"),
+            ("white", "⚪"),
+        ],
+        "Bright Colors": [
+            ("bright_black", "⚫"),
+            ("bright_red", "🔴"),
+            ("bright_green", "🟢"),
+            ("bright_yellow", "🟡"),
+            ("bright_blue", "🔵"),
+            ("bright_magenta", "🟣"),
+            ("bright_cyan", "🔷"),
+            ("bright_white", "⚪"),
+        ],
+        "Special Colors": [
+            ("orange1", "🟠"),
+            ("orange3", "🟠"),
+            ("orange4", "🟠"),
+            ("deep_sky_blue1", "🔷"),
+            ("deep_sky_blue2", "🔷"),
+            ("deep_sky_blue3", "🔷"),
+            ("deep_sky_blue4", "🔷"),
+            ("turquoise2", "🔷"),
+            ("turquoise4", "🔷"),
+            ("steel_blue1", "🔷"),
+            ("steel_blue3", "🔷"),
+            ("chartreuse1", "🟢"),
+            ("chartreuse2", "🟢"),
+            ("chartreuse3", "🟢"),
+            ("chartreuse4", "🟢"),
+            ("gold1", "🟡"),
+            ("gold3", "🟡"),
+            ("rosy_brown", "🔴"),
+            ("indian_red", "🔴"),
+        ],
+    }
+
+    # Suggested colors for each type
+    if color_type == "additions":
+        suggestions = [
+            ("green", "🟢"),
+            ("bright_green", "🟢"),
+            ("chartreuse1", "🟢"),
+            ("green3", "🟢"),
+            ("sea_green1", "🟢"),
+        ]
+        emit_info(
+            "[bold white on green]🎨 Recommended Colors for Additions:[/bold white on green]"
+        )
+        for color, emoji in suggestions:
+            emit_info(
+                f"  [cyan]{color:<16}[/cyan] [white on {color}]■■■■■■■■■■[/white on {color}] {emoji}"
+            )
+    elif color_type == "deletions":
+        suggestions = [
+            ("orange1", "🟠"),
+            ("red", "🔴"),
+            ("bright_red", "🔴"),
+            ("indian_red", "🔴"),
+            ("dark_red", "🔴"),
+        ]
+        emit_info(
+            "[bold white on orange1]🎨 Recommended Colors for Deletions:[/bold white on orange1]"
+        )
+        for color, emoji in suggestions:
+            emit_info(
+                f"  [cyan]{color:<16}[/cyan] [white on {color}]■■■■■■■■■■[/white on {color}] {emoji}"
+            )
+
+    emit_info("\n[bold]🎨 All Available Rich Colors:[/bold]")
+    for category, colors in color_categories.items():
+        emit_info(f"\n[cyan]{category}:[/cyan]")
+        # Display in columns for better readability
+        for i in range(0, len(colors), 4):
+            row = colors[i : i + 4]
+            row_text = "  ".join([f"[{color}]■[/{color}] {color}" for color, _ in row])
+            emit_info(f"  {row_text}")
+
+    emit_info("\n[yellow]Usage:[/yellow] [cyan]/diff {color_type} <color_name>[/cyan]")
+    emit_info("[dim]All diffs use white text on your chosen background colors[/dim]")
+    emit_info("[dim]You can also use hex colors like #ff0000 or rgb(255,0,0)[/dim]")
+
+
 def handle_command(command: str):
     from code_puppy.messaging import emit_error, emit_info, emit_success, emit_warning
+
     _ensure_plugins_loaded()
 
     """
@@ -244,12 +341,14 @@ def handle_command(command: str):
         from code_puppy.agents import get_current_agent
         from code_puppy.command_line.model_picker_completion import get_active_model
         from code_puppy.config import (
+            get_auto_save_session,
             get_compaction_strategy,
             get_compaction_threshold,
             get_openai_reasoning_effort,
             get_owner_name,
             get_protected_token_count,
             get_puppy_name,
+            get_use_dbos,
             get_yolo_mode,
         )
 
@@ -257,6 +356,7 @@ def handle_command(command: str):
         owner_name = get_owner_name()
         model = get_active_model()
         yolo_mode = get_yolo_mode()
+        auto_save = get_auto_save_session()
         protected_tokens = get_protected_token_count()
         compaction_threshold = get_compaction_threshold()
         compaction_strategy = get_compaction_strategy()
@@ -271,6 +371,8 @@ def handle_command(command: str):
 [bold]current_agent:[/bold]         [magenta]{current_agent.display_name}[/magenta]
 [bold]model:[/bold]                 [green]{model}[/green]
 [bold]YOLO_MODE:[/bold]             {"[red]ON[/red]" if yolo_mode else "[yellow]off[/yellow]"}
+[bold]DBOS:[/bold]                  {"[green]enabled[/green]" if get_use_dbos() else "[yellow]disabled[/yellow]"} (toggle: /set enable_dbos true|false)
+[bold]auto_save_session:[/bold]     {"[green]enabled[/green]" if auto_save else "[yellow]disabled[/yellow]"}
 [bold]protected_tokens:[/bold]      [cyan]{protected_tokens:,}[/cyan] recent tokens preserved
 [bold]compaction_threshold:[/bold]     [cyan]{compaction_threshold:.1%}[/cyan] context usage triggers compaction
 [bold]compaction_strategy:[/bold]   [cyan]{compaction_strategy}[/cyan] (summarization or truncation)
@@ -318,6 +420,7 @@ def handle_command(command: str):
             get_current_autosave_session_name,
             rotate_autosave_id,
         )
+
         if len(tokens) == 1 or tokens[1] == "id":
             sid = get_current_autosave_id()
             emit_info(
@@ -363,8 +466,14 @@ def handle_command(command: str):
             )
             return True
         if key:
+            # Check if we're toggling DBOS enablement
+            if key == "enable_dbos":
+                emit_info(
+                    "[yellow]⚠️ DBOS configuration changed. Please restart Code Puppy for this change to take effect.[/yellow]"
+                )
+
             set_config_value(key, value)
-            emit_success(f'🌶 Set {key} = "{value}" in puppy.cfg!')
+            emit_success(f'Set {key} = "{value}" in puppy.cfg!')
         else:
             emit_error("You must supply a key.")
         return True
@@ -619,9 +728,7 @@ def handle_command(command: str):
                     if is_json_agent and hasattr(current_agent, "refresh_config"):
                         current_agent.refresh_config()
                     current_agent.reload_code_generation_agent()
-                    emit_info(
-                        f"Active agent reloaded with pinned model '{model_name}'"
-                    )
+                    emit_info(f"Active agent reloaded with pinned model '{model_name}'")
                 except Exception as reload_error:
                     emit_warning(
                         f"Pinned model applied but reload failed: {reload_error}"
@@ -733,6 +840,7 @@ def handle_command(command: str):
         # Rotate autosave id to avoid overwriting any existing autosave
         try:
             from code_puppy.config import rotate_autosave_id
+
             new_id = rotate_autosave_id()
             autosave_info = f"\n[dim]Autosave session rotated to: {new_id}[/dim]"
         except Exception:
@@ -744,6 +852,10 @@ def handle_command(command: str):
         )
         return True
 
+    if command.startswith("/autosave_load"):
+        # Return a special marker to indicate we need to run async autosave loading
+        return "__AUTOSAVE_LOAD__"
+
     if command.startswith("/truncate"):
         from code_puppy.agents.agent_manager import get_current_agent
 
@@ -786,6 +898,190 @@ def handle_command(command: str):
         )
         return True
 
+    if command.startswith("/diff"):
+        # Handle diff configuration commands
+        from code_puppy.config import (
+            get_diff_addition_color,
+            get_diff_deletion_color,
+            get_diff_highlight_style,
+            set_diff_addition_color,
+            set_diff_deletion_color,
+            set_diff_highlight_style,
+        )
+
+        tokens = command.split()
+
+        if len(tokens) == 1:
+            # Show current diff configuration
+            add_color = get_diff_addition_color()
+            del_color = get_diff_deletion_color()
+
+            emit_info("[bold magenta]🎨 Diff Configuration[/bold magenta]")
+            # Show the actual color pairs being used
+            from code_puppy.tools.file_modifications import _get_optimal_color_pair
+
+            add_fg, add_bg = _get_optimal_color_pair(add_color, "green")
+            del_fg, del_bg = _get_optimal_color_pair(del_color, "orange1")
+            current_style = get_diff_highlight_style()
+            if current_style == "highlighted":
+                emit_info(
+                    f"[bold]Additions:[/bold]       [{add_fg} on {add_bg}]■■■■■■■■■■[/{add_fg} on {add_bg}] {add_color}"
+                )
+                emit_info(
+                    f"[bold]Deletions:[/bold]       [{del_fg} on {del_bg}]■■■■■■■■■■[/{del_fg} on {del_bg}] {del_color}"
+                )
+            if current_style == "text":
+                emit_info(
+                    f"[bold]Additions:[/bold]       [{add_color}]■■■■■■■■■■[/{add_color}] {add_color}"
+                )
+                emit_info(
+                    f"[bold]Deletions:[/bold]       [{del_color}]■■■■■■■■■■[/{del_color}] {del_color}"
+                )
+            emit_info("\n[yellow]Subcommands:[/yellow]")
+            emit_info(
+                "  [cyan]/diff style <style>[/cyan]                 Set diff style (text/highlighted)"
+            )
+            emit_info(
+                "  [cyan]/diff additions <color>[/cyan]             Set addition color (shows options if no color)"
+            )
+            emit_info(
+                "  [cyan]/diff deletions <color>[/cyan]             Set deletion color (shows options if no color)"
+            )
+            emit_info(
+                "  [cyan]/diff show[/cyan]                         Show current configuration with example"
+            )
+
+            if current_style == "text":
+                emit_info(
+                    "\n[dim]Current mode: Plain text diffs (no highlighting)[/dim]"
+                )
+            else:
+                emit_info(
+                    "\n[dim]Current mode: Intelligent color pairs for maximum contrast[/dim]"
+                )
+            return True
+
+        subcmd = tokens[1].lower()
+
+        if subcmd == "style":
+            if len(tokens) == 2:
+                # Show current style
+                current_style = get_diff_highlight_style()
+                emit_info("[bold magenta]🎨 Current Diff Style[/bold magenta]")
+                emit_info(f"Style: {current_style}")
+                emit_info("\n[yellow]Available styles:[/yellow]")
+                emit_info(
+                    "  [cyan]text[/cyan]         - Plain text diffs with no highlighting"
+                )
+                emit_info(
+                    "  [cyan]highlighted[/cyan]   - Intelligent color pairs for maximum contrast"
+                )
+                emit_info("\n[dim]Use '/diff style <style>' to change[/dim]")
+                return True
+            elif len(tokens) != 3:
+                emit_warning("Usage: /diff style <style>")
+                emit_info("[dim]Use '/diff style' to see available styles[/dim]")
+                return True
+
+            new_style = tokens[2].lower()
+            try:
+                set_diff_highlight_style(new_style)
+                emit_success(f"Diff style set to '{new_style}'")
+            except Exception as e:
+                emit_error(f"Failed to set diff style: {e}")
+            return True
+
+        if subcmd == "additions":
+            if len(tokens) == 2:
+                # Show available color options
+                _show_color_options("additions")
+                return True
+            elif len(tokens) != 3:
+                emit_warning("Usage: /diff additions <color>")
+                emit_info("[dim]Use '/diff additions' to see available colors[/dim]")
+                return True
+
+            color = tokens[2]
+            try:
+                set_diff_addition_color(color)
+                emit_success(f"Addition color set to '{color}'")
+            except Exception as e:
+                emit_error(f"Failed to set addition color: {e}")
+            return True
+
+        elif subcmd == "deletions":
+            if len(tokens) == 2:
+                # Show available color options
+                _show_color_options("deletions")
+                return True
+            elif len(tokens) != 3:
+                emit_warning("Usage: /diff deletions <color>")
+                emit_info("[dim]Use '/diff deletions' to see available colors[/dim]")
+                return True
+
+            color = tokens[2]
+            try:
+                set_diff_deletion_color(color)
+                emit_success(f"Deletion color set to '{color}'")
+            except Exception as e:
+                emit_error(f"Failed to set deletion color: {e}")
+            return True
+
+        elif subcmd == "show":
+            # Show current configuration with example
+            from code_puppy.tools.file_modifications import _colorize_diff
+
+            add_color = get_diff_addition_color()
+            del_color = get_diff_deletion_color()
+
+            # Create a simple diff example
+            example_diff = """--- a/example.txt
++++ b/example.txt
+@@ -1,3 +1,4 @@
+ line 1
+-old line 2
++new line 2
+ line 3
++added line 4"""
+
+            current_style = get_diff_highlight_style()
+
+            emit_info("[bold magenta]🎨 Current Diff Configuration[/bold magenta]")
+            emit_info(f"Style: {current_style}")
+
+            if current_style == "highlighted":
+                # Show the actual color pairs being used
+                from code_puppy.tools.file_modifications import _get_optimal_color_pair
+
+                add_fg, add_bg = _get_optimal_color_pair(add_color, "green")
+                del_fg, del_bg = _get_optimal_color_pair(del_color, "orange1")
+                emit_info(
+                    f"Additions: [{add_fg} on {add_bg}]■■■■■■■■■■[/{add_fg} on {add_bg}] {add_color}"
+                )
+                emit_info(
+                    f"Deletions: [{del_fg} on {del_bg}]■■■■■■■■■■[/{del_fg} on {del_bg}] {del_color}"
+                )
+            else:
+                emit_info(f"Additions: {add_color} (plain text mode)")
+                emit_info(f"Deletions: {del_color} (plain text mode)")
+            emit_info(
+                "\n[bold cyan]── DIFF EXAMPLE ───────────────────────────────────[/bold cyan]"
+            )
+
+            # Show the colored example
+            colored_example = _colorize_diff(example_diff)
+            emit_info(colored_example, highlight=False)
+
+            emit_info(
+                "[bold cyan]───────────────────────────────────────────────[/bold cyan]\n"
+            )
+            return True
+
+        else:
+            emit_warning(f"Unknown diff subcommand: {subcmd}")
+            emit_info("Use '/diff' to see available subcommands")
+            return True
+
     if command in ("/exit", "/quit"):
         emit_success("Goodbye!")
         # Signal to the main app that we want to exit
@@ -799,11 +1095,24 @@ def handle_command(command: str):
         try:
             from code_puppy import callbacks
 
+            # Import the special result class for markdown commands
+            try:
+                from code_puppy.plugins.customizable_commands.register_callbacks import (
+                    MarkdownCommandResult,
+                )
+            except ImportError:
+                MarkdownCommandResult = None
+
             results = callbacks.on_custom_command(command=command, name=name)
             # Iterate through callback results; treat str as handled (no model run)
             for res in results:
                 if res is True:
                     return True
+                if MarkdownCommandResult and isinstance(res, MarkdownCommandResult):
+                    # Special case: markdown command that should be processed as input
+                    # Replace the command with the markdown content and let it be processed
+                    # This is handled by the caller, so return the content as string
+                    return res.content
                 if isinstance(res, str):
                     # Display returned text to the user and treat as handled
                     try:
