@@ -157,6 +157,7 @@ def get_config_keys():
         "max_saved_sessions",
         "http2",
         "diff_context_lines",
+        "default_agent",
     ]
     # Add DBOS control key
     default_keys.append("enable_dbos")
@@ -188,11 +189,10 @@ def load_mcp_server_configs():
     Returns a dict mapping names to their URL or config dict.
     If file does not exist, returns an empty dict.
     """
-    from code_puppy.messaging.message_queue import emit_error, emit_system_message
+    from code_puppy.messaging.message_queue import emit_error
 
     try:
         if not pathlib.Path(MCP_SERVERS_FILE).exists():
-            emit_system_message("[dim]No MCP configuration was found[/dim]")
             return {}
         with open(MCP_SERVERS_FILE, "r") as f:
             conf = json.loads(f.read())
@@ -996,3 +996,143 @@ def finalize_autosave_session() -> str:
     """Persist the current autosave snapshot and rotate to a fresh session."""
     auto_save_session_if_enabled()
     return rotate_autosave_id()
+
+
+def get_suppress_thinking_messages() -> bool:
+    """
+    Checks puppy.cfg for 'suppress_thinking_messages' (case-insensitive in value only).
+    Defaults to False if not set.
+    Allowed values for ON: 1, '1', 'true', 'yes', 'on' (all case-insensitive for value).
+    When enabled, thinking messages (agent_reasoning, planned_next_steps) will be hidden.
+    """
+    true_vals = {"1", "true", "yes", "on"}
+    cfg_val = get_value("suppress_thinking_messages")
+    if cfg_val is not None:
+        if str(cfg_val).strip().lower() in true_vals:
+            return True
+        return False
+    return False
+
+
+def set_suppress_thinking_messages(enabled: bool):
+    """Sets the suppress_thinking_messages configuration value.
+
+    Args:
+        enabled: Whether to suppress thinking messages
+    """
+    set_config_value("suppress_thinking_messages", "true" if enabled else "false")
+
+
+def get_suppress_informational_messages() -> bool:
+    """
+    Checks puppy.cfg for 'suppress_informational_messages' (case-insensitive in value only).
+    Defaults to False if not set.
+    Allowed values for ON: 1, '1', 'true', 'yes', 'on' (all case-insensitive for value).
+    When enabled, informational messages (info, success, warning) will be hidden.
+    """
+    true_vals = {"1", "true", "yes", "on"}
+    cfg_val = get_value("suppress_informational_messages")
+    if cfg_val is not None:
+        if str(cfg_val).strip().lower() in true_vals:
+            return True
+        return False
+    return False
+
+
+def set_suppress_informational_messages(enabled: bool):
+    """Sets the suppress_informational_messages configuration value.
+
+    Args:
+        enabled: Whether to suppress informational messages
+    """
+    set_config_value("suppress_informational_messages", "true" if enabled else "false")
+
+
+# API Key management functions
+def get_api_key(key_name: str) -> str:
+    """Get an API key from puppy.cfg.
+
+    Args:
+        key_name: The name of the API key (e.g., 'OPENAI_API_KEY')
+
+    Returns:
+        The API key value, or empty string if not set
+    """
+    return get_value(key_name) or ""
+
+
+def set_api_key(key_name: str, value: str):
+    """Set an API key in puppy.cfg.
+
+    Args:
+        key_name: The name of the API key (e.g., 'OPENAI_API_KEY')
+        value: The API key value (empty string to remove)
+    """
+    set_config_value(key_name, value)
+
+
+def load_api_keys_to_environment():
+    """Load all API keys from .env and puppy.cfg into environment variables.
+
+    Priority order:
+    1. .env file (highest priority) - if present in current directory
+    2. puppy.cfg - fallback if not in .env
+    3. Existing environment variables - preserved if already set
+
+    This should be called on startup to ensure API keys are available.
+    """
+    from pathlib import Path
+
+    api_key_names = [
+        "OPENAI_API_KEY",
+        "GEMINI_API_KEY",
+        "ANTHROPIC_API_KEY",
+        "CEREBRAS_API_KEY",
+        "SYN_API_KEY",
+        "AZURE_OPENAI_API_KEY",
+        "AZURE_OPENAI_ENDPOINT",
+        "OPENROUTER_API_KEY",
+        "ZAI_API_KEY",
+    ]
+
+    # Step 1: Load from .env file if it exists (highest priority)
+    # Look for .env in current working directory
+    env_file = Path.cwd() / ".env"
+    if env_file.exists():
+        try:
+            from dotenv import load_dotenv
+
+            # override=True means .env values take precedence over existing env vars
+            load_dotenv(env_file, override=True)
+        except ImportError:
+            # python-dotenv not installed, skip .env loading
+            pass
+
+    # Step 2: Load from puppy.cfg, but only if not already set
+    # This ensures .env has priority over puppy.cfg
+    for key_name in api_key_names:
+        # Only load from config if not already in environment
+        if key_name not in os.environ or not os.environ[key_name]:
+            value = get_api_key(key_name)
+            if value:
+                os.environ[key_name] = value
+
+
+def get_default_agent() -> str:
+    """
+    Get the default agent name from puppy.cfg.
+
+    Returns:
+        str: The default agent name, or "code-puppy" if not set.
+    """
+    return get_value("default_agent") or "code-puppy"
+
+
+def set_default_agent(agent_name: str) -> None:
+    """
+    Set the default agent name in puppy.cfg.
+
+    Args:
+        agent_name: The name of the agent to set as default.
+    """
+    set_config_value("default_agent", agent_name)
