@@ -4,6 +4,7 @@ import os
 import subprocess
 import sys
 import time
+import traceback
 import webbrowser
 from pathlib import Path
 
@@ -16,10 +17,6 @@ from rich.text import Text
 from code_puppy import __version__, callbacks, plugins
 from code_puppy.agents import get_current_agent
 from code_puppy.command_line.attachments import parse_prompt_attachments
-from code_puppy.command_line.prompt_toolkit_completion import (
-    get_input_with_combined_completion,
-    get_prompt_with_active_model,
-)
 from code_puppy.config import (
     AUTOSAVE_DIR,
     COMMAND_HISTORY_FILE,
@@ -76,6 +73,12 @@ async def main():
         type=str,
         help="Specify which agent to use (e.g., --agent code-puppy)",
     )
+    parser.add_argument(
+        "--model",
+        "-m",
+        type=str,
+        help="Specify which model to use (e.g., --model gpt-5)",
+    )
     parser.add_argument(
         "command", nargs="*", help="Run a single command (deprecated, use -p instead)"
     )
@@ -175,8 +178,49 @@ async def main():
         emit_system_message(f"[bold red]{error_msg}[/bold red]")
         return
 
+    # Early model setting if specified via command line
+    # This happens before ensure_config_exists() to ensure config is set up correctly
+    early_model = None
+    if args.model:
+        early_model = args.model.strip()
+        from code_puppy.config import set_model_name
+
+        set_model_name(early_model)
+
     ensure_config_exists()
 
+    # Load API keys from puppy.cfg into environment variables
+    from code_puppy.config import load_api_keys_to_environment
+
+    load_api_keys_to_environment()
+
+    # Handle model validation from command line (validation happens here, setting was earlier)
+    if args.model:
+        from code_puppy.config import _validate_model_exists
+
+        model_name = args.model.strip()
+        try:
+            # Validate that the model exists in models.json
+            if not _validate_model_exists(model_name):
+                from code_puppy.model_factory import ModelFactory
+
+                models_config = ModelFactory.load_config()
+                available_models = list(models_config.keys()) if models_config else []
+
+                emit_system_message(
+                    f"[bold red]Error:[/bold red] Model '{model_name}' not found"
+                )
+                emit_system_message(f"Available models: {', '.join(available_models)}")
+                sys.exit(1)
+
+            # Model is valid, show confirmation (already set earlier)
+            emit_system_message(f"🎯 Using model: {model_name}")
+        except Exception as e:
+            emit_system_message(
+                f"[bold red]Error validating model:[/bold red] {str(e)}"
+            )
+            sys.exit(1)
+
     # Handle agent selection from command line
     if args.agent:
         from code_puppy.agents.agent_manager import (
@@ -229,9 +273,6 @@ async def main():
 
     # Initialize DBOS if not disabled
     if get_use_dbos():
-        dbos_message = f"Initializing DBOS with database at: {DBOS_DATABASE_URL}"
-        emit_system_message(dbos_message)
-
         dbos_config: DBOSConfig = {
             "name": "dbos-code-puppy",
             "system_database_url": DBOS_DATABASE_URL,
@@ -382,10 +423,9 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
 
     # Check if prompt_toolkit is installed
     try:
-        from code_puppy.messaging import emit_system_message
-
-        emit_system_message(
-            "[dim]Using prompt_toolkit for enhanced tab completion[/dim]"
+        from code_puppy.command_line.prompt_toolkit_completion import (
+            get_input_with_combined_completion,
+            get_prompt_with_active_model,
         )
     except ImportError:
         from code_puppy.messaging import emit_warning
@@ -400,6 +440,10 @@ async def interactive_mode(message_renderer, initial_command: str = None) -> Non
             from code_puppy.messaging import emit_success
 
             emit_success("Successfully installed prompt_toolkit")
+            from code_puppy.command_line.prompt_toolkit_completion import (
+                get_input_with_combined_completion,
+                get_prompt_with_active_model,
+            )
         except Exception as e:
             from code_puppy.messaging import emit_error, emit_warning
 
@@ -741,8 +785,7 @@ def main_entry():
     try:
         asyncio.run(main())
     except KeyboardInterrupt:
-        # Just exit gracefully with no error message
-        callbacks.on_shutdown()
+        print(traceback.format_exc())
         if get_use_dbos():
             DBOS.destroy()
         return 0
