@@ -146,7 +146,11 @@ class ModelFactory:
 
     @staticmethod
     def get_model(model_name: str, config: Dict[str, Any]) -> Any:
-        """Returns a configured model instance based on the provided name and config."""
+        """Returns a configured model instance based on the provided name and config.
+
+        API key validation happens naturally within each model type's initialization,
+        which emits warnings and returns None if keys are missing.
+        """
         model_config = config.get(model_name)
         if not model_config:
             raise ValueError(f"Model '{model_name}' not found in configuration.")
@@ -154,15 +158,27 @@ class ModelFactory:
         model_type = model_config.get("type")
 
         if model_type == "gemini":
-            provider = GoogleProvider(api_key=os.environ.get("GEMINI_API_KEY", ""))
+            api_key = os.environ.get("GEMINI_API_KEY")
+            if not api_key:
+                emit_warning(
+                    f"GEMINI_API_KEY is not set; skipping Gemini model '{model_config.get('name')}'."
+                )
+                return None
 
+            provider = GoogleProvider(api_key=api_key)
             model = GoogleModel(model_name=model_config["name"], provider=provider)
             setattr(model, "provider", provider)
             return model
 
         elif model_type == "openai":
-            provider = OpenAIProvider(api_key=os.environ.get("OPENAI_API_KEY", ""))
+            api_key = os.environ.get("OPENAI_API_KEY")
+            if not api_key:
+                emit_warning(
+                    f"OPENAI_API_KEY is not set; skipping OpenAI model '{model_config.get('name')}'."
+                )
+                return None
 
+            provider = OpenAIProvider(api_key=api_key)
             model = OpenAIChatModel(model_name=model_config["name"], provider=provider)
             if model_name == "gpt-5-codex-api":
                 model = OpenAIResponsesModel(
@@ -184,6 +200,11 @@ class ModelFactory:
 
         elif model_type == "custom_anthropic":
             url, headers, verify, api_key = get_custom_config(model_config)
+            if not api_key:
+                emit_warning(
+                    f"API key is not set for custom Anthropic endpoint; skipping model '{model_config.get('name')}'."
+                )
+                return None
             client = create_async_client(headers=headers, verify=verify)
             anthropic_client = AsyncAnthropic(
                 base_url=url,
@@ -194,6 +215,11 @@ class ModelFactory:
             return AnthropicModel(model_name=model_config["name"], provider=provider)
         elif model_type == "claude_code":
             url, headers, verify, api_key = get_custom_config(model_config)
+            if not api_key:
+                emit_warning(
+                    f"API key is not set for Claude Code endpoint; skipping model '{model_config.get('name')}'."
+                )
+                return None
             client = create_async_client(headers=headers, verify=verify)
             anthropic_client = AsyncAnthropic(
                 base_url=url, http_client=client, auth_token=api_key
@@ -306,6 +332,11 @@ class ModelFactory:
             return zai_model
         elif model_type == "custom_gemini":
             url, headers, verify, api_key = get_custom_config(model_config)
+            if not api_key:
+                emit_warning(
+                    f"API key is not set for custom Gemini endpoint; skipping model '{model_config.get('name')}'."
+                )
+                return None
             os.environ["GEMINI_API_KEY"] = api_key
 
             class CustomGoogleGLAProvider(GoogleProvider):
@@ -337,13 +368,16 @@ class ModelFactory:
                     return profile
 
             url, headers, verify, api_key = get_custom_config(model_config)
+            if not api_key:
+                emit_warning(
+                    f"API key is not set for Cerebras endpoint; skipping model '{model_config.get('name')}'."
+                )
+                return None
             client = create_async_client(headers=headers, verify=verify)
             provider_args = dict(
                 api_key=api_key,
                 http_client=client,
             )
-            if api_key:
-                provider_args["api_key"] = api_key
             provider = ZaiCerebrasProvider(**provider_args)
 
             model = OpenAIChatModel(model_name=model_config["name"], provider=provider)
@@ -362,14 +396,20 @@ class ModelFactory:
                     api_key = os.environ.get(env_var_name)
                     if api_key is None:
                         emit_warning(
-                            f"OpenRouter API key environment variable '{env_var_name}' not found or is empty; proceeding without API key."
+                            f"OpenRouter API key environment variable '{env_var_name}' not found or is empty; skipping model '{model_config.get('name')}'."
                         )
-                    else:
-                        # It's a raw API key value
-                        api_key = api_key_config
+                        return None
+                else:
+                    # It's a raw API key value
+                    api_key = api_key_config
             else:
                 # No API key in config, try to get it from the default environment variable
                 api_key = os.environ.get("OPENROUTER_API_KEY")
+                if api_key is None:
+                    emit_warning(
+                        f"OPENROUTER_API_KEY is not set; skipping OpenRouter model '{model_config.get('name')}'."
+                    )
+                    return None
 
             provider = OpenRouterProvider(api_key=api_key)
 
