@@ -1,7 +1,9 @@
 # agent_tools.py
+import asyncio
 import traceback
-from typing import List
+from typing import List, Set
 
+from dbos import DBOS, SetWorkflowID
 from pydantic import BaseModel
 
 # Import Agent from pydantic_ai to create temporary agents for invocation
@@ -18,6 +20,8 @@ from code_puppy.model_factory import ModelFactory
 from code_puppy.tools.common import generate_group_id
 
 _temp_agent_count = 0
+# Set to track active subagent invocation tasks
+_active_subagent_tasks: Set[asyncio.Task] = set()
 
 
 class AgentInfo(BaseModel):
@@ -160,6 +164,7 @@ def register_invoke_agent(agent):
 
             global _temp_agent_count
             _temp_agent_count += 1
+            subagent_name = f"temp-invoke-agent-{_temp_agent_count}"
             temp_agent = Agent(
                 model=model,
                 instructions=instructions,
@@ -177,14 +182,35 @@ def register_invoke_agent(agent):
                 from pydantic_ai.durable_exec.dbos import DBOSAgent
 
                 dbos_agent = DBOSAgent(
-                    temp_agent, name=f"temp-invoke-agent-{_temp_agent_count}"
+                    temp_agent, name=subagent_name
                 )
                 temp_agent = dbos_agent
 
-            # Run the temporary agent with the provided prompt
-            result = await temp_agent.run(
-                prompt, usage_limits=UsageLimits(request_limit=get_message_limit())
-            )
+            # Run the temporary agent with the provided prompt as an asyncio task
+            if get_use_dbos():
+                with SetWorkflowID(group_id):
+                    task = asyncio.create_task(
+                        temp_agent.run(
+                            prompt, usage_limits=UsageLimits(request_limit=get_message_limit())
+                        )
+                    )
+                    _active_subagent_tasks.add(task)
+            else:
+                task = asyncio.create_task(
+                    temp_agent.run(
+                        prompt, usage_limits=UsageLimits(request_limit=get_message_limit())
+                    )
+                )
+                _active_subagent_tasks.add(task)
+
+            try:
+                result = await task
+            finally:
+                _active_subagent_tasks.discard(task)
+                if task.cancelled():
+                    if get_use_dbos():
+                        DBOS.cancel_workflow(group_id)
+
 
             # Extract the response from the result
             response = result.output
